# !/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Initiate the file."""
__author__ = "Jason Nguyen"
__copyright__ = "Copyright 2020, Jason Nguyen"
__credits__ = ["Jason Nguyen"]
__license__ = "MIT"
__maintainer__ = "Jason Nguyen"
__email__ = "jnguye21@uoguelph.ca"
__status__ = "Planning"
__version__ = "0.0.8.3"

# from .core import main
# from .corpus import Corpus, corn
# __all__ = ["Corpus", "corn", "main"]
