# !/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Common helper functions.

This will represent the bread and butter of the application.
"""


def add_two_numbers(x_val: int, y_val: int) -> int:
    """Just adds two numbers together."""
    return x_val + y_val


def return_string() -> str:
    """Return the string doggo."""
    return "string"
